{% extends 'lara_django_base/base2.html' %} 

{% block head_title %} LARA Chat {% endblock head_title %} 

{% block content %}
    <div class="chat-container">
        <div class="chat-header">
            <h2 class="text-lg font-semibold">LARA Chat</h2>
        </div>
        <div class="chat-messages">
            <!-- Chat messages will be dynamically loaded here -->
        </div>
        <div class="chat-input">
            <input type="text" placeholder="Type your message..." class="border rounded-lg p-2 w-full">
        </div>
    </div>
{% endblock content %}