<!-- dummy profile page -->

<h1>Profile</h1>

<h2>Personal Information</h2>

<p>User logged in : {{ request.user.username }}</p>

Preferred home screen: {{ user.home_screen_url }}

<a href="{% url 'account_logout' %}">Logout</a>