{% load static %}

{% comment %}
  Partial: partials/apps_list.html
  Htmx-only response — renders the full grid of app cards.
  Returned by AppsSelectorPartialView for hx-get requests.
{% endcomment %}

{% for item in apps %}
<div class="lg:col-span-1">
    {% include 'lara_django_base/partials/app_card.html' %}
</div>
{% empty %}
<div class="col-span-3 flex flex-col items-center justify-center py-20 text-base-content/40">
    <svg class="w-16 h-16 mb-4" fill="none" stroke="currentColor" viewBox="0 0 24 24">
        <path stroke-linecap="round" stroke-linejoin="round" stroke-width="1.5"
              d="M9.75 17L9 20l-1 1h8l-1-1-.75-3M3 13h18M5 17h14a2 2 0 002-2V5a2 2 0 00-2-2H5a2 2 0 00-2 2v10a2 2 0 002 2z" />
    </svg>
    <p class="text-lg font-medium">No apps registered yet.</p>
</div>
{% endfor %}
