{% extends "lara_django_base/lara_base.html" %}

{% block title %}About LARA-Django {% endblock %}

{% block dropdown-menu%}{% endblock %}


{% block toolbar %}
<div class="d-flex justify-content-between flex-wrap flex-md-nowrap align-items-center pt-3 pb-2 mb-3 border-bottom">
  {% block title-section %}<h1 class="h2">About LARAsuite Ver. {{ lara_env.lara_version }}
    ({{lara_env.django_environment}}) ...</h1>
  {% endblock %}
</div>
{% endblock %}

{% block content %}

<h6>md</h6>

{% endblock content %}