{% load static %}{% comment %}
Simple icon include template
Usage: {% include 'lara_django_base/includes/icon.html' with name='home' class='w-5 h-5' %}
{% endcomment %}
<svg class="{{ class|default:'w-4 h-4' }}" {% if title %}aria-label="{{ title }}"{% else %}aria-hidden="true"{% endif %}>
    <use href="{% static 'lara_django_base/icons/symbols.svg' %}#{{ name }}"></use>
</svg>
