{# About page content partial — rendered by HtmxResponseMixin. #}

<div class="mx-auto max-w-4xl py-6 px-4 sm:px-6 lg:px-8">

    <h6>{{ lara_env.lara_description }}</h6>
    <ul>
      <li>LARA-django version: <b>{{ lara_env.lara_version }}</b></li>
      <li>LARA authors:
        <ul>
          {% for author in lara_env.lara_authors %}
          <li><b>{{ author }}</b></li>
          {% endfor %}
        </ul>
      </li>
    </ul>

    <h5 class="mt-4">Installed Django Apps</h5>

    <table class="table table-zebra w-full mt-2">
      <thead>
        <tr>
          <th>App Name</th>
          <th>Version</th>
        </tr>
      </thead>
      <tbody>
        {% for app in lara_env.apps %}
        <tr>
          <td>{{ app.name }}</td>
          <td><b>{{ app.version }}</b></td>
        </tr>
        {% endfor %}
      </tbody>
    </table>

    <hr class="my-4" />
    <h3>Acknowledgements — The LARA-django developers thank:</h3>
    <ul class="list-disc ml-6 mt-2">
      <li>the python team</li>
      <li>the whole django team (https://www.djangoproject.com/) for their great tool!</li>
      <li>mptt developers</li>
      <li>the blockly team (https://developers.google.com/blockly/ and https://github.com/google/blockly)
        for their visual programming language</li>
      <li>Vladimir (Владимир) from pyinto (https://github.com/pyinto) for his inspirations on how to
        setup the ordering system and what you can do with CSS and JavaScript</li>
    </ul>

    <hr class="my-4" />
    <h3>System Environment</h3>
    <ul class="list-disc ml-6 mt-2">
      <li>LARA-django environment: <b>{{ lara_env.django_environment }}</b></li>
      <li>django version: <b>{{ lara_env.django_version }}</b></li>
      <li>LARA-django settings: <b>{{ lara_env.django_settings }}</b></li>
    </ul>

</div>
