{% extends "lara_django_base/lara_base.html" %}

{% block title %}About LARA-Django {% endblock %}

{% block dropdown-menu%}{% endblock %}


{% block toolbar %}
<div class="d-flex justify-content-between flex-wrap flex-md-nowrap align-items-center pt-3 pb-2 mb-3 border-bottom">
  {% block title-section %}<h1 class="h2">About LARAsuite Ver. {{ lara_env.lara_version }}
    ({{lara_env.django_environment}}) ...</h1>
  {% endblock %}
</div>
{% endblock %}

{% block content %}

<h6>{{ lara_env.lara_description }}</h6>
<ul>
  <li>LARA-django version: <b>{{ lara_env.lara_version }}</b> </li>
  <li>LARA authors:
    <ul>
      {% for author in lara_env.lara_authors %}
      <li> <b>{{ author }}</b> </li>
      {% endfor %}
    </ul>
  </li>
</ul>

<h5>Installed Django Apps</h5>

<table>
  <tr>
    <th>App Name</th>
    <th>Version</th>
  </tr>
  {% for app in lara_env.apps %}
  <tr>
    <td>{{ app.name }}</td>
    <td><b>{{ app.version }}</b></td>
  </tr>
  {% endfor %}

</table>


<hr />
<h3> Acknowledgements - The LARA-django developers thank:</h3>
<ul>
  <li> the python team </li>
  <li> the whole django team (https://www.djangoproject.com/) for their great tool ! </li>
  <li> mptt developers </li>
  <li> the blockly team (https://developers.google.com/blockly/ and https://github.com/google/blockly) for their visual
    programming language </li>
  <li> Vladimir (Владимир) from pyinto (https://github.com/pyinto) for his inspirations on how to setup the ordering
    system and what you can do with css and JavaSkript </li>
</ul>
<hr />
<h3>System Environment</h3>

<ul>
  <li> LARA-django environment: <b>{{ lara_env.django_environment }}</b> </li>
  <li>django version: <b>{{ lara_env.django_version }}</b> </li>
  <li> LARA-django settings: <b>{{ lara_env.django_settings }}</b> </li>
</ul>
{% endblock %}

