{% extends 'lara_django_base/lara_base.html' %}


{% block content %}

Welcome {{ user.current_user.username }} to the LARA-Django demo site.
logged in : {{  user.authenticated  }}

Preferred home screen: {{ user.home_screen_url }}

{% endblock content %}

