<div class="form-control w-full mb-4">
    <label class="label" for="{{ field.id_for_label }}">
        <span class="label-text font-bold mb-2">{{ field.label }}</span>
    </label>
   
    <div class="ml-1">{{ field }}</div>
    {% if field.help_text %}
    <label class="label">
        <span class="label-text-alt text-gray-500 text-sm ml-2 whitespace-normal break-words">{{ field.help_text }}</span>
    </label>
    {% endif %}
    {% if field.errors %}
    <label class="label">
        <span class="label-text-alt text-error ml-2">{{ field.errors }}</span>
    </label>
    {% endif %}
</div>