{# Contact page content partial — rendered by HtmxResponseMixin. #}

<div class="mx-auto max-w-4xl py-6 px-4 sm:px-6 lg:px-8">
    <h2 class="text-2xl font-semibold mb-4">Contact</h2>
    <p class="text-base-content/70">
        For questions, bug reports, or feature requests please reach out to the
        LARAsuite team.
    </p>
</div>
